/* $Id: xsysaceblkadapter.h,v 1.2 2002/07/19 20:28:12 moleres Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2002 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xsysaceblkadapter.h
*
* The Xilinx System ACE Block Adapter component. This adapter makes the System
* ACE CompactFlash storage device look like a VxWorks block device. In order
* to do this, a BLK_DEV (defined in blkIo.h) structure is allocated and
* populated by the adapter. The BLK_DEV structure describes the device with
* respect to the number and size of blocks, and contains function pointers to
* routines that are used to read and write blocks, reset the storage device,
* and check status of the storage device.
*
* <b>Partitions, or Logical Disks</b>
*
* The user can have multiple partitions, or logical disks, on a single
* CompactFlash storage device. This adapter can be used by the Board Support
* Package (BSP) or application for direct access to the CompactFlash as a
* block device, or it can be used to hook the CompactFlash into a filesystem
* supported by VxWorks (e.g., the DOS filesystem).
*
* <b>Mutual Exclusion</b>
*
* This adapter enforces mutually exclusive access to a particular System ACE
* device. Mutual exclusion is necessary since only one read/write operation
* can be active at any one time to the CompactFlash (because of the shared
* data buffer within the ACE controller).
*
* <b>Polled vs. Interrupt Mode</b>
*
* The System ACE device is used in polled mode for this adapter. The VxWorks
* block device interface does not provide any means for asynchronous callbacks
* or events. Therefore, interrupt mode is not used. VxWorks provides a library
* called Asynchronous I/O (AIO) to aid the application developer when needing
* to use a block device in an asynchronous fashion. The basic idea is to spawn
* a preemptable task that reads and writes the block device, and indicates when
* it is done. Of course the user can implement this scheme without the use of
* AIO as well.
*
* <b>Ioctl</b>
*
* Most ioctl commands are handled by software above this adapter. For example,
* the ioLib.h ioctl commands that pertain to filesystem activities are handled
* by the filesystem software (e.g., dosFs) above this adapter. The only ioctl
* command handled by this adapter is FIODISKFORMAT, used to format the disk.
* This command simply zeroes out every block of a partition. Since only polled
* mode is supported by this adapter, the format can take a while to complete.
*
* <b>Disk Change Notification</b>
*
* The System ACE/CompactFlash does not provide a mechanism for asynchronous
* detection of a disk change (i.e., a disk is removed or inserted). However,
* this adapter does provide a status check function (in the BLK_DEV structure)
* that indicates whether the disk is ready for use or not. VxWorks calls it
* periodically (before each open or creat) to make sure the disk is accessible.
* This method alone still does not indicate whether a new, possibly different,
* disk has been inserted. A disk change could take place between OS calls to
* access the disk. In order to detect a disk change, the user has a few options:
*
*   - The user can manage disk changes by restricting removal or insertion of
*     the disks. This of course depends on a suitable user interface for the
*     application where a person can know when it's OK to remove or insert a
*     disk. The application would have to guarantee that there are no operations
*     in progress and also take care of unmounting and mounting filesystem that
*     may be on top of this block device adapter.
*
*   - The user can manage disk changes when a filesystem is on top of this
*     block device adapter by querying the disk frequently to detect changes.
*     For example, the VxWorks DosFS1.0 provides a "change without warning"
*     option to the filesystem that instructs the filesystem to do just that -
*     query the disk looking for changes. This causes a performance hit since
*     the disk is accessed often to read its volume configuration info, but is
*     an option. The VxWorks DosFS2.0 disk cache behaves in a similar fashion,
*     querying the disk every two seconds and reading a unique DOS volume ID to
*     determine if the disk has been changed.
*
* This type of disk detection is not done by this adapter since the filesystems
* above it usually provide such a mechanism. If it were deemed necessary for
* this adapter to provide such a feature, the adapter could provide an optional
* thread that does nothing but check the status of the CompactFlash every second
* or so. This thread would detect when the disk has been removed or re-inserted,
* and maintain the current state of the disk. It could then use this state to
* prevent access to the disk when it has been removed, or notify the upper layer
* software when the disk has been inserted using the readyChange field of the
* BLK_DEV structure. The spawning of this thread would be optional with an
* argument to the _Init() function of this adapter. The priority and frequency
* at which the thread runs could be tunable by the user.
*
* <b>Note</b>
*
* Xilinx drivers are typically composed of two components, one is the driver
* and the other is the adapter.  The driver is independent of OS and processor
* and is intended to be highly portable.  The adapter is OS-specific and
* facilitates communication between the driver and the OS.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a rpm  07/31/02 First release
* </pre>
*
******************************************************************************/

#ifndef XSYSACE_BLK_ADAPTER_H /* prevent circular inclusions */
#define XSYSACE_BLK_ADAPTER_H /* by using protection macros */

/***************************** Include Files *********************************/

#include "blkIo.h"        /* VxWorks BLK_DEV defines */
#include "xbasic_types.h"
#include "xsysace.h"

/************************** Constant Definitions *****************************/


/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/

STATUS XSysAceBlkAdapter_Init(Xuint16 DeviceId, unsigned MaxPartitions);
unsigned int XSysAceBlkAdapter_GetTotalBlocks(Xuint16 DeviceId);
BLK_DEV *XSysAceBlkAdapter_DevCreate(Xuint16 DeviceId, unsigned PartitionId,
                                     unsigned NumBlks, unsigned Offset);
XSysAce *XSysAceBlkAdapter_GetDriver(Xuint16 DeviceId);


#endif            /* end of protection macro */
